@@ -14,12 +14,12 @@
 #include "command_p.h"
 
 #include "fileoperationspreferences.h"
+#include <utils/applicationstate.h>
 #include "utils/filedialog.h"
 
 #include <Libkleo/Classify>
 #include <Libkleo/Formatting>
 
-#include <KConfigGroup>
 #include <KLocalizedString>
 #include <KSharedConfig>
 
@@ -29,6 +29,8 @@
 #include <QFileInfo>
 #include <QStandardPaths>
 
+#include <gpgme++/context.h>
+
 #include <algorithm>
 #include <memory>
 #include <vector>
@@ -42,18 +44,6 @@ using namespace GpgME;
 namespace
 {
 
-QString getLastUsedExportDirectory()
-{
-    KConfigGroup config{KSharedConfig::openConfig(), "ExportDialog"};
-    return config.readEntry("LastDirectory", QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation));
-}
-
-void updateLastUsedExportDirectory(const QString &path)
-{
-    KConfigGroup config{KSharedConfig::openConfig(), "ExportDialog"};
-    config.writeEntry("LastDirectory", QFileInfo{path}.absolutePath());
-}
-
 QString openPGPCertificateFileExtension()
 {
     return QLatin1String{outputFileExtension(Class::OpenPGP | Class::Ascii | Class::Certificate,
@@ -92,7 +82,7 @@ QString proposeFilename(const Key &key)
     filename = QStringView{u"%1_%2_SECRET"}.arg(name, shortKeyID);
     filename.replace(u'/', u'_');
 
-    return getLastUsedExportDirectory() + u'/' + filename + u'.' + certificateFileExtension(key.protocol());
+    return ApplicationState::lastUsedExportDirectory() + u'/' + filename + u'.' + certificateFileExtension(key.protocol());
 }
 
 QString secretKeyFileFilters(GpgME::Protocol protocol)
@@ -122,7 +112,7 @@ QString requestFilename(const Key &key, const QString &proposedFilename, QWidget
         if (fi.suffix().isEmpty()) {
             filename += u'.' + certificateFileExtension(key.protocol());
         }
-        updateLastUsedExportDirectory(filename);
+        ApplicationState::setLastUsedExportDirectory(filename);
     }
 
     return filename;
@@ -218,6 +208,10 @@ std::unique_ptr<QGpgME::ExportJob> ExportSecretKeyCommand::Private::startExportJ
     std::unique_ptr<QGpgME::ExportJob> exportJob{backend->secretKeyExportJob(armor)};
     Q_ASSERT(exportJob);
 
+    if (key.protocol() == GpgME::CMS) {
+        exportJob->setExportFlags(GpgME::Context::ExportPKCS12);
+    }
+
     connect(exportJob.get(), &QGpgME::ExportJob::result,
             q, [this](const GpgME::Error &err, const QByteArray &keyData) {
                 onExportJobResult(err, keyData);
@@ -241,6 +235,11 @@ std::unique_ptr<QGpgME::ExportJob> ExportSecretKeyCommand::Private::startExportJ
 
 void ExportSecretKeyCommand::Private::onExportJobResult(const Error &err, const QByteArray &keyData)
 {
+    if (err.isCanceled()) {
+        finished();
+        return;
+    }
+
     if (err) {
         showError(err);
         finished();
