@@ -30,23 +30,29 @@
 #include "commands/refreshcertificatecommand.h"
 #include "commands/revokecertificationcommand.h"
 #include "commands/revokeuseridcommand.h"
+#include "commands/setprimaryuseridcommand.h"
 #include "commands/adduseridcommand.h"
 #include "commands/genrevokecommand.h"
 #include "commands/detailscommand.h"
 #include "commands/dumpcertificatecommand.h"
-
+#include "utils/accessibility.h"
 #include "utils/keys.h"
 #include "utils/tags.h"
+#include "view/infofield.h"
 
 #include <Libkleo/Algorithm>
+#include <Libkleo/Compliance>
 #include <Libkleo/Formatting>
 #include <Libkleo/Dn>
 #include <Libkleo/KeyCache>
+#include <Libkleo/KeyHelpers>
 #include <Libkleo/GnuPG>
+#include <Libkleo/NavigatableTreeWidget>
 
 #include <KLocalizedString>
 #include <KMessageBox>
 #include <KSeparator>
+#include <kwidgetsaddons_version.h>
 
 #include <gpgme++/context.h>
 #include <gpgme++/key.h>
@@ -82,172 +88,20 @@ Q_DECLARE_METATYPE(GpgME::UserID)
 
 using namespace Kleo;
 
-static const char *accessibleNameProperty = "_kleo_accessibleName";
-
-class InfoField {
-public:
-    InfoField(const QString &label, QWidget *parent);
-
-    void setValue(const QString &value, const QString &accessibleValue = {});
-    QString value() const;
-
-    void setIcon(const QIcon &icon);
-    void setAction(const QAction *action);
-    void setToolTip(const QString &toolTip);
-    void setVisible(bool visible);
-
-    QLabel *label() const { return mLabel; }
-    QLayout *layout() const { return mLayout; }
-
-private:
-    void onActionChanged();
-
-    QLabel *mLabel = nullptr;
-    QHBoxLayout *mLayout = nullptr;
-    QLabel *mIcon = nullptr;
-    QLabel *mValue = nullptr;
-    QPushButton *mButton = nullptr;
-    const QAction *mAction = nullptr;
-};
-
-InfoField::InfoField(const QString &label, QWidget *parent)
-    : mLabel{new QLabel{label, parent}}
-    , mLayout{new QHBoxLayout}
-    , mIcon{new QLabel{parent}}
-    , mValue{new QLabel{parent}}
-    , mButton{new QPushButton{parent}}
-{
-    mLabel->setBuddy(mValue);
-    mLabel->setTextInteractionFlags(Qt::TextSelectableByMouse);
-    mIcon->setVisible(false);
-    mLayout->addWidget(mIcon);
-    mValue->setTextInteractionFlags(Qt::TextSelectableByMouse);
-    mValue->setFocusPolicy(Qt::TabFocus);
-    mLayout->addWidget(mValue);
-    mButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
-    mButton->setVisible(false);
-    mLayout->addWidget(mButton);
-    mLayout->addStretch();
-}
-
-void InfoField::setValue(const QString &value, const QString &accessibleValue)
-{
-    mValue->setText(value);
-    mValue->setAccessibleName(accessibleValue);
-}
-
-QString InfoField::value() const
-{
-    return mValue->text();
-}
-
-void InfoField::setIcon(const QIcon &icon)
-{
-    if (!icon.isNull()) {
-        const int iconSize = mIcon->style()->pixelMetric(QStyle::PM_SmallIconSize, nullptr, mIcon);
-        mIcon->setPixmap(icon.pixmap(iconSize));
-        mIcon->setVisible(true);
-    } else {
-        mIcon->setVisible(false);
-        mIcon->clear();
-    }
-}
-
-void InfoField::setAction(const QAction *action)
-{
-    if (action == mAction) {
-        return;
-    }
-    if (mAction) {
-        QObject::disconnect(mButton, {}, mAction, {});
-        QObject::disconnect(mAction, {}, mButton, {});
-    }
-    mAction = action;
-    if (mAction) {
-        QObject::connect(mButton, &QPushButton::clicked, action, &QAction::trigger);
-        QObject::connect(mAction, &QAction::changed, mButton, [this]() {
-            onActionChanged();
-        });
-        onActionChanged();
-        mButton->setAccessibleName(mAction->property(accessibleNameProperty).toString());
-        mButton->setVisible(true);
-    } else {
-        mButton->setVisible(false);
-        mButton->setText({});
-        mButton->setIcon({});
-    }
-}
-
-void InfoField::setToolTip(const QString &toolTip)
-{
-    mValue->setToolTip(toolTip);
-}
-
-void InfoField::setVisible(bool visible)
-{
-    mLabel->setVisible(visible);
-    mIcon->setVisible(visible && !mIcon->pixmap(Qt::ReturnByValue).isNull());
-    mValue->setVisible(visible);
-    mButton->setVisible(visible && mAction);
-}
-
-void InfoField::onActionChanged()
-{
-    if (!mAction) {
-        return;
-    }
-    if (mAction->text() != mButton->text()) {
-        mButton->setText(mAction->text());
-    }
-    mButton->setIcon(mAction->icon());
-    if (mAction->toolTip() != mButton->toolTip()) {
-        mButton->setToolTip(mAction->toolTip());
-    }
-    if (mAction->isEnabled() != mButton->isEnabled()) {
-        mButton->setEnabled(mAction->isEnabled());
-    }
-}
-
 namespace
 {
-static bool userHasCertificationKey() {
-    const auto secretKeys = KeyCache::instance()->secretKeys();
-    return Kleo::any_of(secretKeys, [](const auto &k) {
-        return (k.protocol() == GpgME::OpenPGP) && canCreateCertifications(k);
+std::vector<GpgME::UserID> selectedUserIDs(const QTreeWidget *treeWidget) {
+    if (!treeWidget) {
+        return {};
+    }
+
+    std::vector<GpgME::UserID> userIDs;
+    const auto selected = treeWidget->selectedItems();
+    std::transform(selected.begin(), selected.end(), std::back_inserter(userIDs), [](const QTreeWidgetItem *item) {
+        return item->data(0, Qt::UserRole).value<GpgME::UserID>();
     });
+    return userIDs;
 }
-
-class UserIDTable : public QTreeWidget
-{
-    Q_OBJECT
-public:
-    using QTreeWidget::QTreeWidget;
-
-    std::vector<GpgME::UserID> selectedUserIDs() {
-        std::vector<GpgME::UserID> userIDs;
-        const auto selected = selectedItems();
-        std::transform(selected.begin(), selected.end(), std::back_inserter(userIDs), [](const QTreeWidgetItem *item) {
-            return item->data(0, Qt::UserRole).value<GpgME::UserID>();
-        });
-        return userIDs;
-    }
-
-protected:
-    QModelIndex moveCursor(QAbstractItemView::CursorAction cursorAction, Qt::KeyboardModifiers modifiers) override
-    {
-        // make keyboard navigation with Left/Right possible by switching the selection behavior to SelectItems
-        // before calling QTreeWidget::moveCursor, because QTreeWidget::moveCursor ignores MoveLeft/MoveRight
-        // if the selection behavior is SelectRows
-        if ((cursorAction == MoveLeft) || (cursorAction == MoveRight)) {
-            setSelectionBehavior(SelectItems);
-        }
-        const auto result = QTreeWidget::moveCursor(cursorAction, modifiers);
-        if ((cursorAction == MoveLeft) || (cursorAction == MoveRight)) {
-            setSelectionBehavior(SelectRows);
-        }
-        return result;
-    }
-};
 }
 
 class CertificateDetailsWidget::Private
@@ -271,6 +125,7 @@ public:
     void webOfTrustClicked();
     void exportClicked();
     void addUserID();
+    void setPrimaryUserID(const GpgME::UserID &uid = {});
     void changePassphrase();
     void changeExpiration();
     void keysMayHaveChanged();
@@ -312,8 +167,9 @@ private:
     struct UI {
         QWidget *userIDs = nullptr;
         QLabel *userIDTableLabel = nullptr;
-        UserIDTable *userIDTable = nullptr;
+        NavigatableTreeWidget *userIDTable = nullptr;
         QPushButton *addUserIDBtn = nullptr;
+        QPushButton *setPrimaryUserIDBtn = nullptr;
         QPushButton *certifyBtn = nullptr;
         QPushButton *revokeCertificationsBtn = nullptr;
         QPushButton *revokeUserIDBtn = nullptr;
@@ -353,7 +209,7 @@ private:
             userIDTableLabel = new QLabel(i18n("User IDs:"), parent);
             userIDsLayout->addWidget(userIDTableLabel);
 
-            userIDTable = new UserIDTable{parent};
+            userIDTable = new NavigatableTreeWidget{parent};
             userIDTableLabel->setBuddy(userIDTable);
             userIDTable->setAccessibleName(i18n("User IDs"));
             QTreeWidgetItem *__qtreewidgetitem = new QTreeWidgetItem();
@@ -373,6 +229,10 @@ private:
             addUserIDBtn = new QPushButton(i18nc("@action:button", "Add User ID"), parent);
             buttonRow->addWidget(addUserIDBtn);
 
+            setPrimaryUserIDBtn = new QPushButton{i18nc("@action:button", "Flag as Primary"), parent};
+            setPrimaryUserIDBtn->setToolTip(i18nc("@info:tooltip", "Flag the selected user ID as the primary user ID of this key."));
+            buttonRow->addWidget(setPrimaryUserIDBtn);
+
             certifyBtn = new QPushButton(i18nc("@action:button", "Certify User IDs"), parent);
             buttonRow->addWidget(certifyBtn);
 
@@ -426,11 +286,11 @@ private:
                 gridLayout->addLayout(validFromField->layout(), row, 1);
 
                 row++;
-                expiresField = std::make_unique<InfoField>(i18n("Expires:"), parent);
+                expiresField = std::make_unique<InfoField>(i18n("Valid until:"), parent);
                 changeExpirationAction = new QAction{parent};
                 changeExpirationAction->setIcon(QIcon::fromTheme(QStringLiteral("editor")));
-                changeExpirationAction->setToolTip(i18nc("@info:tooltip", "Change the expiration date"));
-                changeExpirationAction->setProperty(accessibleNameProperty, i18nc("@action:button", "Change expiration"));
+                changeExpirationAction->setToolTip(i18nc("@info:tooltip", "Change the end of the validity period"));
+                Kleo::setAccessibleName(changeExpirationAction, i18nc("@action:button", "Change Validity"));
                 expiresField->setAction(changeExpirationAction);
                 gridLayout->addWidget(expiresField->label(), row, 0);
                 gridLayout->addLayout(expiresField->layout(), row, 1);
@@ -441,7 +301,7 @@ private:
                     copyFingerprintAction = new QAction{parent};
                     copyFingerprintAction->setIcon(QIcon::fromTheme(QStringLiteral("edit-copy")));
                     copyFingerprintAction->setToolTip(i18nc("@info:tooltip", "Copy the fingerprint to the clipboard"));
-                    copyFingerprintAction->setProperty(accessibleNameProperty, i18nc("@action:button", "Copy fingerprint"));
+                    Kleo::setAccessibleName(copyFingerprintAction, i18nc("@action:button", "Copy fingerprint"));
                     fingerprintField->setAction(copyFingerprintAction);
                 }
                 gridLayout->addWidget(fingerprintField->label(), row, 0);
@@ -452,7 +312,7 @@ private:
                 showIssuerCertificateAction = new QAction{parent};
                 showIssuerCertificateAction->setIcon(QIcon::fromTheme(QStringLiteral("dialog-information")));
                 showIssuerCertificateAction->setToolTip(i18nc("@info:tooltip", "Show the issuer certificate"));
-                showIssuerCertificateAction->setProperty(accessibleNameProperty, i18nc("@action:button", "Show certificate"));
+                Kleo::setAccessibleName(showIssuerCertificateAction, i18nc("@action:button", "Show certificate"));
                 smimeIssuerField->setAction(showIssuerCertificateAction);
                 gridLayout->addWidget(smimeIssuerField->label(), row, 0);
                 gridLayout->addLayout(smimeIssuerField->layout(), row, 1);
@@ -482,6 +342,8 @@ private:
 
             mainLayout->addWidget(smimeAddressList);
 
+            mainLayout->addStretch();
+
             {
                 auto buttonRow = new QHBoxLayout;
 
@@ -531,6 +393,8 @@ CertificateDetailsWidget::Private::Private(CertificateDetailsWidget *qq)
             q, [this]() { updateUserIDActions(); });
     connect(ui.addUserIDBtn, &QPushButton::clicked,
             q, [this]() { addUserID(); });
+    connect(ui.setPrimaryUserIDBtn, &QPushButton::clicked,
+            q, [this]() { setPrimaryUserID(); });
     connect(ui.revokeUserIDBtn, &QPushButton::clicked,
             q, [this]() { revokeSelectedUserID(); });
     connect(ui.changePassphraseBtn, &QPushButton::clicked,
@@ -569,10 +433,17 @@ void CertificateDetailsWidget::Private::setupCommonProperties()
     const bool isOpenPGP = key.protocol() == GpgME::OpenPGP;
     const bool isSMIME = key.protocol() == GpgME::CMS;
     const bool isOwnKey = key.hasSecret();
+    const auto isLocalKey = !isRemoteKey(key);
+    const auto keyCanBeCertified = Kleo::canBeCertified(key);
 
     // update visibility of UI elements
     ui.userIDs->setVisible(isOpenPGP);
     ui.addUserIDBtn->setVisible(isOwnKey);
+#ifdef QGPGME_SUPPORTS_SET_PRIMARY_UID
+    ui.setPrimaryUserIDBtn->setVisible(isOwnKey);
+#else
+    ui.setPrimaryUserIDBtn->setVisible(false);
+#endif
     // ui.certifyBtn->setVisible(true); // always visible (for OpenPGP keys)
     // ui.webOfTrustBtn->setVisible(true); // always visible (for OpenPGP keys)
     ui.revokeCertificationsBtn->setVisible(Kleo::Commands::RevokeCertificationCommand::isSupported());
@@ -591,24 +462,26 @@ void CertificateDetailsWidget::Private::setupCommonProperties()
     }
     // ui.fingerprintField->setVisible(true); // always visible
     ui.smimeIssuerField->setVisible(isSMIME);
-    ui.complianceField->setVisible(Kleo::gnupgIsDeVsCompliant());
+    ui.complianceField->setVisible(DeVSCompliance::isCompliant());
     ui.trustedIntroducerField->setVisible(isOpenPGP); // may be hidden again by setupPGPProperties()
 
     ui.smimeRelatedAddresses->setVisible(isSMIME);
     ui.smimeAddressList->setVisible(isSMIME);
 
-    // ui.moreDetailsBtn->setVisible(true); // always visible
+    ui.moreDetailsBtn->setVisible(isLocalKey);
     ui.trustChainDetailsBtn->setVisible(isSMIME);
-    // ui.refreshBtn->setVisible(true); // always visible
+    ui.refreshBtn->setVisible(isLocalKey);
     ui.changePassphraseBtn->setVisible(isSecretKeyStoredInKeyRing(key));
-    // ui.exportBtn->setVisible(true); // always visible
+    ui.exportBtn->setVisible(isLocalKey);
     ui.genRevokeBtn->setVisible(isOpenPGP && isOwnKey);
 
     // update availability of buttons
     const auto userCanSignUserIDs = userHasCertificationKey();
     ui.addUserIDBtn->setEnabled(canBeUsedForSecretKeyOperations(key));
-    ui.certifyBtn->setEnabled(userCanSignUserIDs);
-    ui.revokeCertificationsBtn->setEnabled(userCanSignUserIDs);
+    ui.setPrimaryUserIDBtn->setEnabled(false); // requires a selected user ID
+    ui.certifyBtn->setEnabled(isLocalKey && keyCanBeCertified && userCanSignUserIDs);
+    ui.webOfTrustBtn->setEnabled(isLocalKey);
+    ui.revokeCertificationsBtn->setEnabled(userCanSignUserIDs && isLocalKey);
     ui.revokeUserIDBtn->setEnabled(false); // requires a selected user ID
     ui.changeExpirationAction->setEnabled(canBeUsedForSecretKeyOperations(key));
     ui.changePassphraseBtn->setEnabled(isSecretKeyStoredInKeyRing(key));
@@ -616,19 +489,25 @@ void CertificateDetailsWidget::Private::setupCommonProperties()
 
     // update values of protocol-independent UI elements
     ui.validFromField->setValue(Formatting::creationDateString(key), Formatting::accessibleCreationDate(key));
-    ui.expiresField->setValue(Formatting::expirationDateString(key, i18nc("Expires", "never")),
-                              Formatting::accessibleExpirationDate(key, i18nc("Expires", "never")));
+    ui.expiresField->setValue(Formatting::expirationDateString(key, i18nc("Valid until:", "unlimited")),
+                              Formatting::accessibleExpirationDate(key));
     ui.fingerprintField->setValue(Formatting::prettyID(key.primaryFingerprint()),
                                   Formatting::accessibleHexID(key.primaryFingerprint()));
-    if (Kleo::gnupgIsDeVsCompliant()) {
+    if (DeVSCompliance::isCompliant()) {
         ui.complianceField->setValue(Kleo::Formatting::complianceStringForKey(key));
     }
 }
 
 void CertificateDetailsWidget::Private::updateUserIDActions()
 {
-    const auto userIDs = ui.userIDTable->selectedUserIDs();
-    ui.revokeUserIDBtn->setEnabled((userIDs.size() == 1) && canCreateCertifications(key) && canRevokeUserID(userIDs.front()));
+    const auto userIDs = selectedUserIDs(ui.userIDTable);
+    const auto singleUserID = userIDs.size() == 1 ? userIDs.front() : GpgME::UserID{};
+    const bool isPrimaryUserID = !singleUserID.isNull() && (ui.userIDTable->selectedItems().front() == ui.userIDTable->topLevelItem(0));
+    ui.setPrimaryUserIDBtn->setEnabled(!singleUserID.isNull() //
+                                       && !isPrimaryUserID //
+                                       && !Kleo::isRevokedOrExpired(singleUserID) //
+                                       && canBeUsedForSecretKeyOperations(key));
+    ui.revokeUserIDBtn->setEnabled(!singleUserID.isNull() && canCreateCertifications(key) && canRevokeUserID(singleUserID));
 }
 
 void CertificateDetailsWidget::Private::setUpUserIDTable()
@@ -697,40 +576,43 @@ void CertificateDetailsWidget::Private::setUpSMIMEAdressList()
         }
     }
 
-    if (key.numUserIDs() < 2) {
-        return;
+    if (key.numUserIDs() > 1) {
+        // iterate over the secondary user IDs
+#ifdef USE_RANGES
+        for (const auto uids = key.userIDs(); const auto &uid : std::ranges::subrange(std::next(uids.begin()), uids.end())) {
+#else
+        const auto uids = key.userIDs();
+        for (auto it = std::next(uids.begin()); it != uids.end(); ++it) {
+            const auto &uid = *it;
+#endif
+            const auto name = Kleo::Formatting::prettyName(uid);
+            const auto email = Kleo::Formatting::prettyEMail(uid);
+            QString itemText;
+            if (name.isEmpty() && !email.isEmpty()) {
+                // skip email addresses already listed in email attribute field
+                if (emailField && email == emailField->value()) {
+                    continue;
+                }
+                itemText = email;
+            } else {
+                // S/MIME certificates sometimes contain urls where both
+                // name and mail is empty. In that case we print whatever
+                // the uid is as name.
+                //
+                // Can be ugly like (3:uri24:http://ca.intevation.org), but
+                // this is better then showing an empty entry.
+                itemText = QString::fromUtf8(uid.id());
+            }
+            // avoid duplicate entries in the list
+            if (ui.smimeAddressList->findItems(itemText, Qt::MatchExactly).empty()) {
+                ui.smimeAddressList->addItem(itemText);
+            }
+        }
     }
 
-    // iterate over the secondary user IDs
-#ifdef USE_RANGES
-    for (const auto uids = key.userIDs(); const auto &uid : std::ranges::subrange(std::next(uids.begin()), uids.end())) {
-#else
-    const auto uids = key.userIDs();
-    for (auto it = std::next(uids.begin()); it != uids.end(); ++it) {
-        const auto &uid = *it;
-#endif
-        const auto name = Kleo::Formatting::prettyName(uid);
-        const auto email = Kleo::Formatting::prettyEMail(uid);
-        QString itemText;
-        if (name.isEmpty() && !email.isEmpty()) {
-            // skip email addresses already listed in email attribute field
-            if (emailField && email == emailField->value()) {
-                continue;
-            }
-            itemText = email;
-        } else {
-            // S/MIME certificates sometimes contain urls where both
-            // name and mail is empty. In that case we print whatever
-            // the uid is as name.
-            //
-            // Can be ugly like (3:uri24:http://ca.intevation.org), but
-            // this is better then showing an empty entry.
-            itemText = QString::fromUtf8(uid.id());
-        }
-        // avoid duplicate entries in the list
-        if (ui.smimeAddressList->findItems(itemText, Qt::MatchExactly).empty()) {
-            ui.smimeAddressList->addItem(itemText);
-        }
+    if (ui.smimeAddressList->count() == 0) {
+        ui.smimeRelatedAddresses->setVisible(false);
+        ui.smimeAddressList->setVisible(false);
     }
 }
 
@@ -740,13 +622,26 @@ void CertificateDetailsWidget::Private::revokeUserID(const GpgME::UserID &userId
         "@info",
         "<para>Do you really want to revoke the user ID<nl/><emphasis>%1</emphasis> ?</para>",
         QString::fromUtf8(userId.id()));
-    auto confirmButton = KStandardGuiItem::yes();
+    auto confirmButton = KStandardGuiItem::ok();
     confirmButton.setText(i18nc("@action:button", "Revoke User ID"));
     confirmButton.setToolTip({});
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+    const auto choice = KMessageBox::questionTwoActions(
+#else
     const auto choice = KMessageBox::questionYesNo(
-        q->window(), message, i18nc("@title:window", "Confirm Revocation"),
-        confirmButton, KStandardGuiItem::cancel(), {}, KMessageBox::Notify | KMessageBox::WindowModal);
+#endif
+        q->window(),
+        message,
+        i18nc("@title:window", "Confirm Revocation"),
+        confirmButton,
+        KStandardGuiItem::cancel(),
+        {},
+        KMessageBox::Notify | KMessageBox::WindowModal);
+#if KWIDGETSADDONS_VERSION >= QT_VERSION_CHECK(5, 100, 0)
+    if (choice != KMessageBox::ButtonCode::PrimaryAction) {
+#else
     if (choice != KMessageBox::Yes) {
+#endif
         return;
     }
 
@@ -764,7 +659,7 @@ void CertificateDetailsWidget::Private::revokeUserID(const GpgME::UserID &userId
 
 void CertificateDetailsWidget::Private::revokeSelectedUserID()
 {
-    const auto userIDs = ui.userIDTable->selectedUserIDs();
+    const auto userIDs = selectedUserIDs(ui.userIDTable);
     if (userIDs.size() != 1) {
         return;
     }
@@ -817,7 +712,7 @@ void CertificateDetailsWidget::Private::refreshCertificate()
 
 void CertificateDetailsWidget::Private::certifyUserIDs()
 {
-    const auto userIDs = ui.userIDTable->selectedUserIDs();
+    const auto userIDs = selectedUserIDs(ui.userIDTable);
     auto cmd = userIDs.empty() ? new Kleo::Commands::CertifyCertificateCommand{key} //
                                : new Kleo::Commands::CertifyCertificateCommand{userIDs};
     QObject::connect(cmd, &Kleo::Commands::CertifyCertificateCommand::finished,
@@ -831,7 +726,7 @@ void CertificateDetailsWidget::Private::certifyUserIDs()
 
 void CertificateDetailsWidget::Private::revokeCertifications()
 {
-    const auto userIDs = ui.userIDTable->selectedUserIDs();
+    const auto userIDs = selectedUserIDs(ui.userIDTable);
     auto cmd = userIDs.empty() ? new Kleo::Commands::RevokeCertificationCommand{key} //
                                : new Kleo::Commands::RevokeCertificationCommand{userIDs};
     QObject::connect(cmd, &Kleo::Command::finished,
@@ -869,6 +764,28 @@ void CertificateDetailsWidget::Private::addUserID()
     cmd->start();
 }
 
+void CertificateDetailsWidget::Private::setPrimaryUserID(const GpgME::UserID &uid)
+{
+    auto userId = uid;
+    if (userId.isNull()) {
+        const auto userIDs = selectedUserIDs(ui.userIDTable);
+        if (userIDs.size() != 1) {
+            return;
+        }
+        userId = userIDs.front();
+    }
+
+    auto cmd = new Kleo::Commands::SetPrimaryUserIDCommand(userId);
+    QObject::connect(cmd, &Kleo::Commands::SetPrimaryUserIDCommand::finished, q, [this]() {
+        ui.userIDTable->setEnabled(true);
+        // the Flag As Primary button will be updated by the key update
+        updateKey();
+    });
+    ui.userIDTable->setEnabled(false);
+    ui.setPrimaryUserIDBtn->setEnabled(false);
+    cmd->start();
+}
+
 namespace
 {
 void ensureThatKeyDetailsAreLoaded(GpgME::Key &key)
@@ -897,11 +814,29 @@ void CertificateDetailsWidget::Private::showTrustChainDialog()
 
 void CertificateDetailsWidget::Private::userIDTableContextMenuRequested(const QPoint &p)
 {
-    const auto userIDs = ui.userIDTable->selectedUserIDs();
+    const auto userIDs = selectedUserIDs(ui.userIDTable);
     const auto singleUserID = (userIDs.size() == 1) ? userIDs.front() : GpgME::UserID{};
+#ifdef QGPGME_SUPPORTS_SET_PRIMARY_UID
+    const bool isPrimaryUserID = !singleUserID.isNull() && (ui.userIDTable->selectedItems().front() == ui.userIDTable->topLevelItem(0));
+#endif
     const bool canSignUserIDs = userHasCertificationKey();
+    const auto isLocalKey = !isRemoteKey(key);
+    const auto keyCanBeCertified = Kleo::canBeCertified(key);
 
     auto menu = new QMenu(q);
+#ifdef QGPGME_SUPPORTS_SET_PRIMARY_UID
+    if (key.hasSecret()) {
+        auto action = menu->addAction(QIcon::fromTheme(QStringLiteral("favorite")),
+                                      i18nc("@action:inmenu", "Flag as Primary User ID"),
+                                      q, [this, singleUserID]() {
+                                          setPrimaryUserID(singleUserID);
+                                      });
+        action->setEnabled(!singleUserID.isNull() //
+                           && !isPrimaryUserID //
+                           && !Kleo::isRevokedOrExpired(singleUserID) //
+                           && canBeUsedForSecretKeyOperations(key));
+    }
+#endif
     {
         const auto actionText = userIDs.empty() ? i18nc("@action:inmenu", "Certify User IDs...")
                                                 : i18ncp("@action:inmenu", "Certify User ID...", "Certify User IDs...", userIDs.size());
@@ -910,7 +845,7 @@ void CertificateDetailsWidget::Private::userIDTableContextMenuRequested(const QP
                                       q, [this]() {
                                           certifyUserIDs();
                                       });
-        action->setEnabled(canSignUserIDs);
+        action->setEnabled(isLocalKey && keyCanBeCertified && canSignUserIDs);
     }
     if (Kleo::Commands::RevokeCertificationCommand::isSupported()) {
         const auto actionText = userIDs.empty() ? i18nc("@action:inmenu", "Revoke Certifications...")
@@ -920,7 +855,7 @@ void CertificateDetailsWidget::Private::userIDTableContextMenuRequested(const QP
                                       q, [this]() {
                                           revokeCertifications();
                                       });
-        action->setEnabled(canSignUserIDs);
+        action->setEnabled(isLocalKey && canSignUserIDs);
     }
 #ifdef MAILAKONADI_ENABLED
     if (key.hasSecret()) {
@@ -950,20 +885,15 @@ void CertificateDetailsWidget::Private::userIDTableContextMenuRequested(const QP
 
 void CertificateDetailsWidget::Private::showMoreDetails()
 {
-    ui.moreDetailsBtn->setEnabled(false);
     if (key.protocol() == GpgME::CMS) {
         auto cmd = new Kleo::Commands::DumpCertificateCommand(key);
-        connect(cmd, &Kleo::Commands::DumpCertificateCommand::finished,
-                q, [this]() {
-                    ui.moreDetailsBtn->setEnabled(true);
-                });
+        cmd->setParentWidget(q);
         cmd->setUseDialog(true);
         cmd->start();
     } else {
         QScopedPointer<SubKeysDialog> dlg(new SubKeysDialog(q));
         dlg->setKey(key);
         dlg->exec();
-        ui.moreDetailsBtn->setEnabled(true);
     }
 }
 
@@ -1140,7 +1070,7 @@ void CertificateDetailsWidget::Private::showIssuerCertificate()
         KMessageBox::error(q, i18n("The issuer certificate could not be found locally."));
         return;
     }
-    auto cmd = new Kleo::Commands::DetailsCommand(parentKeys.front(), nullptr);
+    auto cmd = new Kleo::Commands::DetailsCommand(parentKeys.front());
     cmd->setParentWidget(q);
     cmd->start();
 }
@@ -1225,5 +1155,5 @@ GpgME::Key CertificateDetailsWidget::key() const
     return d->key;
 }
 
-#include "certificatedetailswidget.moc"
 #include "moc_certificatedetailswidget.cpp"
+
