@@ -7,8 +7,7 @@
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
-#ifndef __KLEOPATRA_UISERVER_ECHOCOMMAND_H__
-#define __KLEOPATRA_UISERVER_ECHOCOMMAND_H__
+#pragma once
 
 #include "assuancommand.h"
 #include <QObject>
@@ -66,4 +65,3 @@ private:
 
 }
 
-#endif /* __KLEOPATRA_UISERVER_ECHOCOMMAND_H__ */
