@@ -1,5 +1,4 @@
-#ifndef VIEW_NULLPINWIDGET_H
-#define VIEW_NULLPINWIDGET_H
+#pragma once
 /*  view/nullpinwidget.h
 
     This file is part of Kleopatra, the KDE keymanager
@@ -12,11 +11,6 @@
 
 class QPushButton;
 
-namespace GpgME
-{
-    class Error;
-} // namespace GpgME
-
 namespace Kleo
 {
 
@@ -41,4 +35,3 @@ private:
 
 } // namespace Kleo
 
-#endif // VIEW_NULLPINWIDGET_H
